"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _storyboard = require("storyboard");

var _kebabCase = _interopRequireDefault(require("kebab-case"));

var _semver = _interopRequireDefault(require("semver"));

var _readSpecs = require("./utils/readSpecs");

var _removeInternalLinks = _interopRequireDefault(require("./utils/removeInternalLinks"));

var _writeSpecs = _interopRequireDefault(require("./utils/writeSpecs"));

var _shell = require("./utils/shell");

var _promises = require("./utils/promises");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const PASS_THROUGH_OPTS = ['production', 'noLockfile', 'pureLockfile', 'frozenLockfile'];

const run = async opts => {
  const {
    src,
    ignoreSrc,
    link: linkPattern
  } = opts;
  const production = opts.production || process.env.NODE_ENV === 'production'; // Almost nothing to do when using yarn workspaces ;)

  if (opts.workspaces) {
    _storyboard.mainStory.info('Using yarn workspaces...');

    await (0, _shell.exec)('yarn install');
    return;
  } // Proceed, the old way


  const allSpecs = await (0, _readSpecs.readAllSpecs)(src, ignoreSrc);
  const pkgNames = Object.keys(allSpecs);
  const allRemovedDepsByPackage = {};
  const allRemovedDepsByPackageAndType = {}; // Pass 0: register all subpackages (yarn link) [PARALLEL]

  _storyboard.mainStory.info(`${_storyboard.chalk.bold('PASS 0:')} registering all subpackages...`);

  await (0, _promises.runInParallel)(pkgNames, async pkgName => {
    if (pkgName === _readSpecs.ROOT_PACKAGE) return;
    const {
      displayName,
      pkgPath
    } = allSpecs[pkgName];

    _storyboard.mainStory.info(`  - ${_storyboard.chalk.cyan.bold(displayName)}`);

    await (0, _shell.exec)('yarn link', {
      cwd: pkgPath,
      logLevel: 'trace',
      errorLogLevel: 'info' // reduce yarn's log level (stderr) when subpackage is already registered

    });
  }); // Pass 1: install external deps for all subpackages [PARALLEL]

  _storyboard.mainStory.info(`${_storyboard.chalk.bold('PASS 1:')} installing external dependencies...`);

  const installer = async pkgName => {
    // if (pkgName === ROOT_PACKAGE) return;
    const {
      displayName,
      pkgPath,
      specPath,
      specs: prevSpecs
    } = allSpecs[pkgName];

    _storyboard.mainStory.info(`  - ${_storyboard.chalk.cyan.bold(displayName)}`); // Rewrite package.json without own/linked packages, install, and revert changes


    let fModified = false;

    try {
      const {
        nextSpecs,
        allRemovedPackages,
        removedPackagesByType
      } = (0, _removeInternalLinks.default)(prevSpecs, pkgNames, linkPattern);
      allRemovedDepsByPackage[pkgName] = allRemovedPackages;
      allRemovedDepsByPackageAndType[pkgName] = removedPackagesByType;

      if (nextSpecs !== prevSpecs) {
        (0, _writeSpecs.default)(specPath, nextSpecs);
        fModified = true;
      }

      let cmd = 'yarn install';
      PASS_THROUGH_OPTS.forEach(key => {
        if (opts[key]) cmd += ` --${(0, _kebabCase.default)(key)}`;
      });
      await (0, _shell.exec)(cmd, {
        cwd: pkgPath,
        logLevel: 'trace'
      });
    } finally {
      if (prevSpecs != null && fModified) (0, _writeSpecs.default)(specPath, prevSpecs);
    }
  };

  if (opts.parallel) {
    await (0, _promises.runInParallel)(pkgNames, installer, {
      waitForAllToResolve: true
    });
  } else {
    await (0, _promises.runInSeries)(pkgNames, installer);
  } // Pass 2: link internal and user-specified deps [PARALLEL]


  _storyboard.mainStory.info(`${_storyboard.chalk.bold('PASS 2:')} Installing all internal dependencies...`);

  await (0, _promises.runInParallel)(pkgNames, async pkgName => {
    const allRemovedPackages = allRemovedDepsByPackage[pkgName];
    const removedPackagesByType = allRemovedDepsByPackageAndType[pkgName];
    const packagesToLink = Object.keys(allRemovedPackages);
    const {
      displayName,
      pkgPath
    } = allSpecs[pkgName];
    await (0, _promises.runInParallel)(packagesToLink, async depName => {
      if (production && isPureDevDependency(removedPackagesByType, depName)) {
        return;
      }

      _storyboard.mainStory.info(`  - ${_storyboard.chalk.cyan.bold(displayName)} -> ${_storyboard.chalk.cyan.bold(depName)}`);

      const depVersionRange = allRemovedPackages[depName];
      const depSpecs = allSpecs[depName]; // might not exist, if it's a custom link

      const depActualVersion = depSpecs ? depSpecs.specs.version : null;

      if (depActualVersion && !_semver.default.satisfies(depActualVersion, depVersionRange)) {
        _storyboard.mainStory.warn(`    Warning: ${_storyboard.chalk.cyan.bold(`${depName}@${depActualVersion}`)} ` + `does not satisfy specified range: ${_storyboard.chalk.cyan.bold(depVersionRange)}`);
      }

      await (0, _shell.exec)(`yarn link ${depName}`, {
        cwd: pkgPath,
        logLevel: 'trace'
      });
    });
  });
};

const isPureDevDependency = (deps, depName) => !(deps.dependencies && deps.dependencies[depName] || deps.optionalDependencies && deps.optionalDependencies[depName] || deps.peerDependencies && deps.peerDependencies[depName]);

var _default = run;
exports.default = _default;