"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _storyboard = require("storyboard");

var _readSpecs = require("./utils/readSpecs");

var _git = require("./utils/git");

/* eslint-disable no-console */
const run = async opts => {
  (0, _storyboard.config)({
    filter: '-*'
  });
  const lastTag = await gitStatus();
  await subpackageStatus(opts, lastTag);
  console.log('');
};

const gitStatus = async () => {
  console.log('');
  console.log('* Git status:');
  console.log('');

  try {
    const branch = await (0, _git.gitCurBranch)();
    console.log(`    - Current branch: ${_storyboard.chalk.cyan.bold(branch)}`);
  } catch (err) {
    console.log(`    - ${_storyboard.chalk.red.bold('Could not be determined')} (is this a git repo?)`);
  }

  let lastTag;

  try {
    lastTag = await (0, _git.gitLastTag)();
    console.log(`    - Last tag: ${lastTag != null ? _storyboard.chalk.cyan.bold(lastTag) : _storyboard.chalk.yellow.bold('NONE YET')}`);
  } catch (err) {
    /* ignore */
  }

  try {
    const uncommitted = await (0, _git.gitUncommittedChanges)();
    console.log(`    - Uncommitted changes: ${uncommitted !== '' ? _storyboard.chalk.yellow.bold('YES') : _storyboard.chalk.cyan.bold('no')}`);
  } catch (err) {
    /* ignore */
  }

  try {
    const unpulled = await (0, _git.gitUnpulledChanges)();
    console.log(`    - Unpulled changes: ${unpulled !== '0' ? _storyboard.chalk.yellow.bold('YES') : _storyboard.chalk.cyan.bold('no')}`);
  } catch (err) {
    console.log(`    - Unpulled changes: ${_storyboard.chalk.yellow.bold('UNKNOWN (no upstream?)')}`);
  }

  return lastTag;
};

const subpackageStatus = async (opts, lastTag) => {
  const {
    src,
    ignoreSrc
  } = opts;
  let allSpecs;

  try {
    allSpecs = await (0, _readSpecs.readAllSpecs)(src, ignoreSrc);
  } catch (err) {
    if (err.message === 'INVALID_DIR_NAME') {
      console.error(`INVALID_DIR_NAME - ${err.details}`);
    }

    throw err;
  }

  const pkgNames = Object.keys(allSpecs);
  console.log('');
  console.log(`* Subpackage status: [${_storyboard.chalk.cyan.bold(pkgNames.length)} package/s, incl. root]`);
  console.log('');
  console.log(_storyboard.chalk.gray('    Name                                     Version        Private Changes Dependencies'));

  for (let i = 0; i < pkgNames.length; i++) {
    const pkgName = pkgNames[i];
    const {
      pkgPath,
      specs
    } = allSpecs[pkgName];
    let name = pkgName === _readSpecs.ROOT_PACKAGE ? 'Root' : pkgName;
    name = field(name, 40);
    if (pkgName === _readSpecs.ROOT_PACKAGE) name = _storyboard.chalk.italic(name);

    const version = _storyboard.chalk.cyan.bold(field(specs.version, 14));

    const isPrivate = specs.private ? _storyboard.chalk.cyan.bold(field('yes', 7)) : _storyboard.chalk.yellow.bold(field('NO', 7));
    let changes;

    if (pkgName !== _readSpecs.ROOT_PACKAGE) {
      const diff = await (0, _git.gitDiffSinceIn)(lastTag, pkgPath);
      changes = diff !== '' ? _storyboard.chalk.yellow.bold(field(String(diff.split('\n').length), 7)) : _storyboard.chalk.gray(field('-', 7));
    } else {
      changes = _storyboard.chalk.gray(field('N/A', 7));
    }

    const {
      dependencies,
      devDependencies
    } = specs;
    const numDeps = Object.keys(dependencies || {}).length;
    const numDevDeps = Object.keys(devDependencies || {}).length;
    let deps = `${_storyboard.chalk.cyan.bold(numDeps)}`;
    if (numDevDeps) deps += ` (+ ${_storyboard.chalk.cyan.bold(numDevDeps)} dev)`;
    console.log(`    ${name} ${version} ${isPrivate} ${changes} ${deps}`);
  }
};

const field = (str = '', n) => {
  if (str.length > n) return `${str.slice(0, n - 1)}…`;
  let out = str; // inefficient, slow, etc. but doesn't matter in this case, and easy to read

  while (out.length < n) out += ' ';

  return out;
};

var _default = run;
exports.default = _default;