"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.exec = exports.mv = exports.cp = void 0;

var _path = _interopRequireDefault(require("path"));

var _shelljs = _interopRequireDefault(require("shelljs"));

var _split = _interopRequireDefault(require("split"));

var _execa = _interopRequireDefault(require("execa"));

var _storyboard = require("storyboard");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-underscore-dangle */
const cp = (src, dst, {
  story = _storyboard.mainStory
} = {}) => {
  story.debug(`Copying ${_storyboard.chalk.cyan.bold(src)} -> ${_storyboard.chalk.cyan.bold(dst)}...`);

  _shelljs.default.cp('-rf', _path.default.normalize(src), _path.default.normalize(dst));
};

exports.cp = cp;

const mv = (src, dst, {
  story = _storyboard.mainStory
} = {}) => {
  story.debug(`Moving ${_storyboard.chalk.cyan.bold(src)} -> ${_storyboard.chalk.cyan.bold(dst)}...`);

  _shelljs.default.mv('-rf', _path.default.normalize(src), _path.default.normalize(dst));
};

exports.mv = mv;

const exec = async (cmd, {
  story = _storyboard.mainStory,
  storySrc,
  createChildStory = true,
  logLevel = 'info',
  errorLogLevel = 'error',
  ignoreErrorCode = false,
  bareLogs = false,
  cwd
} = {}) => {
  let title = `Run cmd ${_storyboard.chalk.green.bold(cmd)}`;
  if (cwd) title += ` at ${_storyboard.chalk.green(cwd)}`;
  const ownStory = createChildStory ? story.child({
    title,
    level: logLevel
  }) : story || _storyboard.mainStory;

  try {
    return await _exec(cmd, {
      cwd,
      story: ownStory,
      storySrc,
      errorLogLevel,
      bareLogs,
      ignoreErrorCode
    });
  } finally {
    if (createChildStory) ownStory.close();
  }
};

exports.exec = exec;

const _exec = async (cmd, {
  cwd,
  story,
  storySrc,
  errorLogLevel,
  bareLogs,
  ignoreErrorCode
}) => {
  try {
    const prefix = bareLogs ? '' : '| ';
    const src = storySrc || cmd.split(' ')[0].slice(0, 10);

    const child = _execa.default.shell(cmd, {
      cwd: cwd || '.',
      // Workaround for Node.js bug: https://github.com/nodejs/node/issues/10836
      // See also: https://github.com/yarnpkg/yarn/issues/2462
      stdio: process.platform === 'win32' ? ['ignore', 'pipe', 'pipe'] : undefined
    });

    child.stdout.pipe((0, _split.default)()).on('data', line => {
      story.info(src, `${prefix}${line}`);
    });
    child.stderr.pipe((0, _split.default)()).on('data', line => {
      if (line) story[errorLogLevel](src, `${prefix}${line}`);
    });
    const {
      code,
      stdout,
      stderr
    } = await child;

    if (code !== 0 && !ignoreErrorCode) {
      throw execError(cmd, cwd, code, stdout, stderr);
    }

    return {
      code,
      stdout,
      stderr
    };
  } catch (err) {
    if (err.code && ignoreErrorCode) {
      const {
        code,
        stdout,
        stderr
      } = err;
      return {
        code,
        stdout,
        stderr
      };
    }

    const err2 = execError(cmd, cwd, err.code, err.stdout, err.stderr);
    story[errorLogLevel](err2.message);
    throw err2;
  }
};

const execError = (cmd, cwd, code, stdout, stderr) => {
  const errorMsg = `Command '${cmd}' failed ${code != null ? `[${code}]` : ''} at ${cwd || "'.'"}`;
  const err = new Error(errorMsg);
  err.code = code;
  err.stdout = stdout;
  err.stderr = stderr;
  return err;
};